/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.common;

// Kelp imports
import org.enhydra.kelp.common.node.OtterNode;

// Standard imports
import java.util.Vector;
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class PropUtil {

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public PropUtil() {}

    /**
     * Method declaration
     *
     *
     * @param s
     * @param def
     *
     * @return
     *
     * @see
     */
    public static int stringToInt(String s, int def) {
        int i = def;

        if (s != null) {
            if (s.trim().length() > 0) {
                try {
                    i = Integer.parseInt(s.trim());
                } catch (Exception e) {
                    i = def;
                }
            }
        }
        return i;
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @return
     *
     * @see
     */
    public static String booleanToString(boolean b) {
        String flag = b ? "true" : "false";

        return flag;
    }

    /**
     * Method declaration
     *
     *
     * @param s
     * @param def
     *
     * @return
     *
     * @see
     */
    public static boolean stringToBoolean(String s, boolean def) {
        boolean b = def;

        if (s != null) {
            if (s.trim().length() > 0) {
                b = s.trim().equalsIgnoreCase("true");
            }
        }
        return b;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param lengthName
     * @param columnName1
     * @param columnName2
     *
     * @return
     *
     * @see
     */
    public static String[][] getArrayProperty(OtterNode node,
                                              String lengthName,
                                              String columnName1,
                                              String columnName2) {
        String[][] array = new String[0][0];
        String     columnValue1 = null;
        String     columnValue2 = null;
        Vector     valueVector = new Vector();
        int        length = 0;

        length = stringToInt(node.getProperty(lengthName), 100);
        for (int i = 0; i < length; i++) {
            columnValue1 = node.getProperty(columnName1 + "." + i);
            columnValue2 = node.getProperty(columnName2 + "." + i);
            if (columnValue1 != null && columnValue2 != null) {
                if (columnValue1.trim().length() > 0
                        && columnValue2.trim().length() > 0) {

                    // note: new 'row' object is needed for each call to addElement
                    String[] row = new String[2];

                    row[0] = columnValue1;
                    row[1] = columnValue2;
                    valueVector.addElement(row);
                }
            }
        }
        length = valueVector.size();
        node.setProperty(lengthName, length);    // update to valid length
        String[] row = new String[2];

        array = new String[length][2];
        for (int i = 0; i < length; i++) {
            row = (String[]) valueVector.elementAt(i);
            array[i][0] = row[0];
            array[i][1] = row[1];
        }
        return array;
    }

    /**
     * Method declaration
     *
     *
     * @param map
     *
     * @see
     */
    public static void putArrayProperty(OtterNode node, String lengthName,
                                        String columnName1,
                                        String columnName2,
                                        String[][] array) {
        int    rawLength = 0;
        int    newLength = 0;
        String columnValue1 = new String();
        String columnValue2 = new String();

        rawLength = stringToInt(node.getProperty(lengthName), 100);
        for (int i = 0; i < rawLength; i++) {
            node.setProperty(columnName1 + "." + i, null);
            node.setProperty(columnName2 + "." + i, null);
        }
        rawLength = array.length;
        for (int i = 0; i < rawLength; i++) {
            columnValue1 = array[i][0];
            columnValue2 = array[i][1];
            if (columnValue1.trim().length() > 0
                    && columnValue2.trim().length() > 0) {
                node.setProperty(columnName1 + "." + newLength, columnValue1);
                node.setProperty(columnName2 + "." + newLength, columnValue2);
                newLength++;
            }
        }
        node.setProperty(lengthName, newLength);    // update to valid length
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param propertyName
     * @param inFilename
     *
     * @see
     */
    public static void putFileRelativeToProject(OtterNode node,
                                                String propertyName,
                                                String inFilename) {
        String outFilename = compressPathRelativeToProject(node, inFilename);

        node.setProperty(propertyName, outFilename);
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param propertyName
     *
     * @return
     *
     * @see
     */
    public static File getFileRelativeToProject(OtterNode node,
                                                String propertyName) {
        File   file = null;
        String filename = node.getProperty(propertyName);

        if (filename != null) {
            filename = PropUtil.expandPathRelativeToProject(node, filename);
            file = new File(filename);
        }
        return file;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param inPath
     *
     * @return
     *
     * @see
     */
    public static String compressPathRelativeToProject(OtterNode node,
            String inPath) {
        StringBuffer outPath = new StringBuffer();
        String absolutePath = null;
        if (inPath.startsWith(".")) {
            outPath.append(inPath);
        } else {
            File   file = new File(node.getProject().getFilePath());
            String projectParent = file.getParent();
            if (projectParent.endsWith(File.separator)) {
              projectParent = projectParent.substring(0, projectParent.length()-1);
            }

            file = new File(inPath);
            absolutePath = file.getAbsolutePath().trim();
            if (absolutePath.length() > inPath.trim().length()) {
               outPath.append(inPath);
                // Not a valid path.
            } else {
                if (absolutePath.startsWith(projectParent)) {
                    if (absolutePath.length() == projectParent.length()) {
                        outPath.append(".");
                        outPath.append(File.separator);
                    } else {
                        outPath.append(".");
                        outPath.append(File.separator);
                        outPath.append(absolutePath.substring(
                            projectParent.length() + 1));
                    }
                } else {
                  outPath.append(absolutePath);
                }
            }
        }
        // Return with separators set to / for any OS
        return outPath.toString().replace('\\','/');
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param inPath
     *
     * @return
     *
     * @see
     */
    public static String expandPathRelativeToProject(OtterNode node,
            String inPath) {
        StringBuffer outPath = new StringBuffer();

        // Set separators to current OS
        inPath = inPath.replace('\\', File.separatorChar);
        inPath = inPath.replace('/', File.separatorChar);

        if (inPath.startsWith(".")) {

            // expand path relative to the project.
            File file = new File(node.getProject().getFilePath());

            outPath.append(file.getParent());
            if ((! file.getParent().endsWith(File.separator)) &&
                (! inPath.substring(2).startsWith(File.separator))) {
              outPath.append(File.separator);
            }
            outPath.append(inPath.substring(2));
        } else {
          outPath.append(inPath);
        }
        return outPath.toString();
    }

}
