/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.common.bridge;

// XMLC imports
//import org.enhydra.xml.xmlc.compiler.CodeGenerator;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.XMLCException;

// Standard imports
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

public class GeneratorV3 implements Generator {

  private Object generator = null;
  private Class genClass = null;
  private Object metaData = null;

  public GeneratorV3(MetaDataHandler handler,
                     XMLCDocument xmlcDoc,
                     PrintWriter verboseOutput)
                     throws XMLCException {
    Constructor[] cons = new Constructor[0];
    Constructor   constructor = null;
    Class[]       params = null;
    XMLCException xe = null;
    Object compileParams = handler.getMetaData();

    metaData = compileParams;
     try {
       genClass =
           Class.forName("org.enhydra.xml.xmlc.compiler.CodeGenerator");    // 3.0
       cons = genClass.getConstructors();
       for (int i = 0; i < cons.length; i++) {
           params = cons[i].getParameterTypes();

            // stop on non-default constructors.
            if (params.length == 3) {
                constructor = cons[i];
                break;
            }
        }
        Object[] values = new Object[3];

            values[0] = metaData;
            values[1] = xmlcDoc;
            values[2] = verboseOutput;
            generator = constructor.newInstance(values);
        } catch (java.lang.reflect.InvocationTargetException e) {
            if (e.getTargetException() instanceof XMLCException) {
                throw (XMLCException) e.getTargetException();
            } else {
                xe = new XMLCException(e.getTargetException().toString());
                e.getTargetException().printStackTrace();
                throw xe;
            }
        } catch (Exception e) {
            xe = new XMLCException(e.toString());
            e.printStackTrace();
            throw xe;
        }
  }

  public void generateJavaSource(PrintWriter writer)
      throws XMLCException {
     // codeGen.generateJavaSource(node.getOptions(), null);
      Method[] meths = genClass.getMethods();
      Method   generate = null;
      Class[]  paramTypes = null;
      XMLCException xe = null;

      for (int i = 0; i < meths.length; i++) {
          if (meths[i].getName().equals("generateJavaSource")) {
              generate = meths[i];
              break;
          }
      }
      if (generate != null) {
          paramTypes = generate.getParameterTypes();
          Object paramValues[] = new Object[paramTypes.length];

          paramValues[0] = metaData;
          paramValues[1] = writer;
          try {
              generate.invoke(generator, paramValues);
          } catch (java.lang.reflect.InvocationTargetException e) {
              e.printStackTrace();
              if (e.getTargetException() instanceof XMLCException) {
                  throw (XMLCException) e.getTargetException();
              } else {
                  xe =
                      new XMLCException(e.getTargetException().toString());
                  throw xe;
              }
          } catch (Exception e) {
              xe = new XMLCException(e.toString());
              e.printStackTrace();
              throw xe;
          }
      }
   }
}
