/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.common.wizard;

import org.enhydra.kelp.common.node.OtterSelectionNode;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.DefaultListModel;

import java.awt.Insets;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;

import java.io.File;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class SelectionPanel extends JPanel {
    public JLabel              labelAvailable;
    public JLabel              labelSelected;


    private OtterSelectionNode[]  nodes          = null;
    private LocalButtonListener   buttonListener = null;

    private JScrollPane         scrollAvailable;
    private JScrollPane         scrollSelected;
    private JList               listAvailable;
    private JList               listSelected;
    private JButton             buttonAdd;
    private JButton             buttonAddAll;
    private JButton             buttonRemove;
    private JButton             buttonRemoveAll;
    private GridBagLayout       layoutSelect;
    private JPanel              panelSelect;
    private JPanel              panelOption;
    private DefaultListModel    modelAvailable = new DefaultListModel();
    private DefaultListModel    modelSelected = new DefaultListModel();
    private BorderLayout layoutMain;
    private GridBagLayout layoutOption;
    private JCheckBox checkShowFull;
    private Vector selectionPanelListeners = new Vector();
    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public SelectionPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    // /
    // /
    // event methods
  public synchronized void addSelectionPanelListener(SelectionPanelListener l) {
    if (! selectionPanelListeners.contains(l)) {
       selectionPanelListeners.addElement(l);
    }
  }

  public synchronized void removeSelectionPanelListener(SelectionPanelListener l) {
    if (selectionPanelListeners.contains(l)) {
      selectionPanelListeners.removeElement(l);
    }
  }



    /**
     * Method declaration
     *
     *
     * @see
     */
    protected void fillLists() {
        int        count = getNodes().length;
        OtterSelectionNode source = null;
        if (getModelSelected().getSize() > 0) {
          getModelSelected().clear();
        }
        if (getModelAvailable().getSize() > 0) {
          getModelAvailable().clear();
        }
        for (int i = 0; i < count; i++) {
            LocalListNode listNode = null;
            source = getNodes()[i];
            listNode = new LocalListNode(source.getFilePath());

            if (source.isSelected()) {
               getModelSelected().addElement(listNode);
            } else {
               getModelAvailable().addElement(listNode);
            }
        }
    }




    // /
    // /

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected DefaultListModel getModelSelected() {
        return modelSelected;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected DefaultListModel getModelAvailable() {
        return modelAvailable;
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterSelectionNode[] getNodes() {
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param source
     *
     * @see
     */
    public void setNodes(OtterSelectionNode[] n) {
        nodes = n;
        fillLists();
        fireSelectionEvent();
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
        buttonListener = new LocalButtonListener();

        buttonAdd.addActionListener(buttonListener);
        buttonAddAll.addActionListener(buttonListener);
        buttonRemove.addActionListener(buttonListener);
        buttonRemoveAll.addActionListener(buttonListener);
        checkShowFull.addActionListener(buttonListener);
        checkShowFull.setSelected(false);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
        layoutSelect =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());

        panelSelect =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                            JPanel.class.getName());
        panelOption =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                            JPanel.class.getName());

        scrollAvailable =
            (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                            JScrollPane.class.getName());
        scrollSelected =
            (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                            JScrollPane.class.getName());
        labelAvailable =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        labelSelected =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        listAvailable = (JList) Beans.instantiate(getClass().getClassLoader(),
                                                  JList.class.getName());
        listSelected = (JList) Beans.instantiate(getClass().getClassLoader(),
                                                 JList.class.getName());
        buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        buttonAddAll =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonRemove =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonRemoveAll =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());

        layoutMain = (BorderLayout) Beans.instantiate(getClass().getClassLoader(), BorderLayout.class.getName());
        layoutOption = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());

    checkShowFull = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    listAvailable.setModel(modelAvailable);
        listSelected.setModel(modelSelected);
        labelAvailable.setText("Available Pages in Project");
        labelSelected.setText("Selected Pages to Compile:");
        buttonAdd.setText(">");
        buttonAddAll.setText(">>");
        buttonRemove.setText("<");
        buttonRemoveAll.setText("<<");
        scrollAvailable.setMaximumSize(new Dimension(200, 200));
        scrollAvailable.setPreferredSize(new Dimension(200, 200));
        scrollSelected.setPreferredSize(new Dimension(200, 200));
    checkShowFull.setText("Show full file path");
    scrollAvailable.getViewport().add(listAvailable);
        scrollSelected.getViewport().add(listSelected);

        panelSelect.setLayout(layoutSelect);
        panelSelect.add(labelAvailable,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 2, 5), 0, 0));
        panelSelect.add(scrollAvailable,
                 new GridBagConstraints(0, 1, 1, 4, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.NONE,
                                        new Insets(3, 5, 5, 5), 250, 250));
        panelSelect.add(buttonAdd,
                 new GridBagConstraints(1, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(8, 4, 4, 4), 20, 0));
        panelSelect.add(buttonAddAll,
                 new GridBagConstraints(1, 2, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(8, 4, 4, 4), 6, 0));
        panelSelect.add(buttonRemove,
                 new GridBagConstraints(1, 3, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.SOUTH,
                                        GridBagConstraints.NONE,
                                        new Insets(8, 4, 4, 4), 14, 0));
        panelSelect.add(buttonRemoveAll,
                 new GridBagConstraints(1, 4, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(8, 4, 4, 4), 6, 0));
        panelSelect.add(labelSelected,
                 new GridBagConstraints(2, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 2, 5), 0, 0));
        panelSelect.add(scrollSelected,
                 new GridBagConstraints(2, 1, 1, 4, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.VERTICAL,
                                        new Insets(3, 5, 5, 5), 250, 250));

         panelOption.setLayout(layoutOption);
         panelOption.add(checkShowFull, new GridBagConstraints(0, 0, 1, 1, 0.5, 0.5
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));




        this.setMinimumSize(new Dimension(400, 200));
        this.setPreferredSize(new Dimension(620, 300));
        this.setLayout(layoutMain);
        this.add(panelSelect, BorderLayout.CENTER);
        this.add(panelOption, BorderLayout.SOUTH);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void addSelected() {
        Object[] elements = listAvailable.getSelectedValues();

        for (int i = 0; i < elements.length; i++) {
           LocalListNode listNode = (LocalListNode) elements[i];
            modelSelected.addElement(listNode);
            modelAvailable.removeElement(listNode);

            for (int j = 0; j < getNodes().length; j++) {
                if (getNodes()[j].getFilePath().equals(listNode.getFullname())) {
                    getNodes()[j].setSelected(true);
                }
            }
        }
        fireSelectionEvent();
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void addAllElements() {
        for (int i = 0; i < modelAvailable.size(); i++) {
            modelSelected.addElement(modelAvailable.elementAt(i));
        }

        modelAvailable.removeAllElements();

        for (int i = 0; i < getNodes().length; i++) {
            getNodes()[i].setSelected(true);
        }
        fireSelectionEvent();
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void removeSelected() {
        Object[] elements = listSelected.getSelectedValues();

        for (int i = 0; i < elements.length; i++) {
            LocalListNode listNode = (LocalListNode) elements[i];
            modelAvailable.addElement(listNode);
            modelSelected.removeElement(listNode);

            for (int j = 0; j < getNodes().length; j++) {
                if (getNodes()[j].getFilePath().equals(listNode.getFullname())) {
                    getNodes()[j].setSelected(false);
                }
            }
        }
        fireSelectionEvent();
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void removeAllElements() {
        for (int i = 0; i < modelSelected.size(); i++) {
            modelAvailable.addElement(modelSelected.elementAt(i));
        }

        modelSelected.removeAllElements();

        for (int i = 0; i < getNodes().length; i++) {
            getNodes()[i].setSelected(false);
        }
        fireSelectionEvent();
    }

    private void fireSelectionEvent() {
        SelectionPanelListener listener = null;
        for (int i = 0 ; i < selectionPanelListeners.size() ; i++ ) {
          listener = (SelectionPanelListener) selectionPanelListeners.elementAt(i);
          listener.onSelection(
             new SelectionPanelEvent(this, modelAvailable, modelSelected));
        }
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

        /**
         * Method declaration
         *
         *
         * @param event
         *
         * @see
         */
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonAdd) {
                addSelected();
            } else if (source == buttonAddAll) {
                addAllElements();
            } else if (source == buttonRemove) {
                removeSelected();
            } else if (source == buttonRemoveAll) {
                removeAllElements();
            } else if (source == checkShowFull) {
                checkShowFull.isSelected();
                fillLists();
            }

        }

  }


    class LocalListNode {
       private String full = new String();

       public LocalListNode(String fullName) {
         full = fullName;
       }

       public String toString() {
          String s = full;
          if (! checkShowFull.isSelected()) {
              File f = new File(full);
              s = f.getName();
          }
          return s;
       }

       public String getFullname() {
         return full;
       }
    }
}
