/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id$
 */
package org.enhydra.kelp.common.wizard.conf;

// Kelp imports
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.FileUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.wizard.AbstractBuildThread;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.capture.Writer;
import org.enhydra.kelp.common.capture.WriteListener;

// Standard imports
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;

/**
 * CompileThread calls XMLC once for each selected HTML file.
 *
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
public class ConfBuildThread extends AbstractBuildThread {
    private File startMultiServerDirectory = null;

    /**
     * Method declaration
     *
     *
     * @param in
     *
     * @return
     *
     * @see
     */
    public static File createPolicyFile(String path) {
        File in = new File(path);
        File out = null;

        if (in.exists()) {
            in.delete();
            in = null;
        }
        try {
            out = new File(path);
            out.getParentFile().mkdirs();
            out.createNewFile();
            PrintWriter writer = new PrintWriter(new FileWriter(out));

            writer.println("grant {");
            writer.println("permission java.security.AllPermission;");
            writer.println("};");
            writer.flush();
            writer.close();
            out = null;
            out = new File(path);
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
        return out;
    }

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public ConfBuildThread() {
        super();
    }

    /**
     * Method declaration
     *
     *
     * @param d
     *
     * @see
     */
    public void setStartMultiServerDirectory(File d) {
        startMultiServerDirectory = d;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getStartMultiServerDirectory() {
        return startMultiServerDirectory;
    }

    /**
     *
     * @param listeners
     */
    public ConfBuildThread(WriteListener[] listeners) {
        super(listeners);
    }

    /**
     * Constructor declaration
     *
     *
     * @param listeners
     * @param buffer
     *
     * @see
     */
    public ConfBuildThread(WriteListener[] listeners, boolean buffer) {
        super(listeners, buffer);
    }

    /**
     * Loop through all selected conf.in files and copy them
     * to the conf directory and do a search and replace using
     * the project conf table.  Write out exceptions to any
     * registered listeners.
     */
    public void make() {
        if (!KelpVersion.getEnhydraVersion().startsWith("3")) {
            makePolicy();
        }
        makeDeploymentDesc();
        if (getProject().isConfStartupSelected()) {
            makeStartup();
        }
        if (getNodes().length > 0) {
            makeSelected();
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private File createPolicy() {
        StringBuffer policyPath = new StringBuffer();
        File         policy = null;

        policyPath.append(getProject().getConfStartupConf());
        policyPath.append(File.separator);
        policyPath.append("java.policy");
        policy = ConfBuildThread.createPolicyFile(policyPath.toString());
        return policy;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void makePolicy() {

        // Find all images in project and copy using
        // the mapping table.
        OtterTextFileNode node = getProject().getFirstPolicy();

        if (node == null) {
            File f = createPolicy();
            getTraceWriter().println("Creating policy file: ");
            getTraceWriter().println("    " + f.getAbsolutePath());
        } else {
            File source = null;
            File dest = null;

            source = new File(node.getFilePath());
            dest = new File(getProject().getConfStartupConf(),
                            source.getName());
            try {
                FileUtil.copyFile(source, dest);
                getTraceWriter().println("Copying policy file from: ");
                getTraceWriter().println("    " + source.getAbsolutePath());
                getTraceWriter().println("to: ");
                getTraceWriter().println("    " + dest.getAbsolutePath());
            } catch (java.io.IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void makeDeploymentDesc() {

        // Find all images in project and copy using
        // the mapping table.
        OtterTextFileNode[] descs = getProject().getAllDeploymentDescs();
        File                source = null;
        File                dest = null;
        String              sourcePath = null;
        String              rawFile = null;
        String              outPath = null;

        for (int i = 0; i < descs.length; i++) {
            source = new File(descs[i].getFilePath());
            sourcePath = getProject().getSourcePathOf(descs[i]);
            rawFile = (new File(descs[i].getFilePath())).getAbsolutePath();
            outPath = rawFile.substring(sourcePath.length());
            dest = new File(getProject().getClassOutputPath() + outPath);
            try {
                FileUtil.copyFile(source, dest);
                getTraceWriter().println("Copying deployment descriptor from: ");
                getTraceWriter().println("    " + source.getAbsolutePath());
                getTraceWriter().println("to: ");
                getTraceWriter().println("    " + dest.getAbsolutePath());
            } catch (java.io.IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void makeStartup() {
        if (getProject().isConfStartupJava()) {
            OtterNodeFactory factory = null;
            StartMultiServer start = null;

            factory = getProject().getNodeFactory();
            start = new StartMultiServer(getProject());
            start.setDirectory(getStartMultiServerDirectory());
            try {
                start.create();
                factory.createJavaFileNode(getProject(), getProject(),
                                           start.getFile().getAbsolutePath());
                getTraceWriter().println("Creating startup class: ");
                getTraceWriter().println("   "
                                         + start.getFile().getAbsolutePath());
            } catch (java.io.IOException e) {
                e.printStackTrace();
            }
        }
        if (getProject().isConfStartupRun()) {
            getProject().configureRunClass();
        }
        getTraceWriter().println("--- startup configuration complete ---");
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void makeSelected() {
        int count = getNodes().length;

        for (int i = 0; i < count; i++) {
            OtterTemplateNode templateNode =
                (OtterTemplateNode) getNodes()[i];

            if (!isFresh()) {
                break;
            }
            try {
                if (templateNode.isSelected() && isFresh()) {
                    File sourceDoc = new File(templateNode.getFilePath());

                    if (sourceDoc.exists()) {
                        if (isPrintProgress()) {
                            getTraceWriter().println("Processing ");
                            getTraceWriter().println("  "
                                                     + sourceDoc.getAbsolutePath());
                        }
                        processNode(templateNode);
                    } else {
                        if (isPrintProgress()) {
                            getTraceWriter().println("Source document not found: "
                                                     + sourceDoc.getAbsolutePath());
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
                if (isPrintProgress()) {
                    getTraceWriter().println("");
                }
            }
        }
        if (isFresh()) {
            if (isPrintProgress()) {
                printResults();
            }
            printExceptions();
            addNodesToProject();
            if (isPrintProgress()) {
                getTraceWriter().println("");
                getTraceWriter().println("--- template processing complete ---");
                getTraceWriter().println("");
            }
        } else {
            printExceptions();
            getTraceWriter().println("");
            getTraceWriter().println("--- Processing interrupted ---");
            getTraceWriter().println("");
        }
    }

    /**
     * Determine what files were created and write out the
     * compilation results to any registered listeners.
     */
    public void printResults() {
        int eCount = 0;
        int totalCount = getNodes().length;
        int selectCount = 0;

        getTraceWriter().println("");
        getTraceWriter().println("Gathering results...");
        for (int i = 0; i < totalCount; i++) {
            if (getNodes()[i].isSelected()
                    && getNodes()[i].getException() != null) {
                eCount++;
            }
        }
        sleepInSeconds(0.1);
        for (int i = 0; i < totalCount; i++) {
            if (getNodes()[i].isSelected()) {
                selectCount++;
            }
        }
        if (eCount < selectCount) {
            getTraceWriter().println("");
            getTraceWriter().println("New conf files:");
            for (int i = 0; i < totalCount; i++) {
                OtterTemplateNode templateNode =
                    (OtterTemplateNode) getNodes()[i];

                sleepInSeconds(0.1);
                if (!isFresh()) {
                    break;
                }
                if (templateNode.isSelected()
                        && templateNode.getOutputFile().exists()) {
                    getTraceWriter().println(" "
                                             + templateNode.getOutputFile().getAbsolutePath());
                }
            }
        }
    }

    /**
     */
    public void addNodesToProject() {
        int eCount = 0;
        int totalCount = getNodes().length;
        int selectCount = 0;

        for (int i = 0; i < totalCount; i++) {
            if (getNodes()[i].isSelected()
                    && getNodes()[i].getException() != null) {
                eCount++;
            }
        }
        for (int i = 0; i < totalCount; i++) {
            if (getNodes()[i].isSelected()) {
                selectCount++;
            }
        }
        if (eCount < selectCount) {
            for (int i = 0; i < totalCount; i++) {
                OtterTemplateNode templateNode =
                    (OtterTemplateNode) getNodes()[i];

                if (!isFresh()) {
                    break;
                }
                if (templateNode.isSelected()
                        && templateNode.getOutputFile().exists()) {
                    addOutputNode(templateNode);
                }
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @param confNode
     *
     * @see
     */
    private void addOutputNode(OtterTemplateNode templateNode) {
        OtterNodeFactory factory = null;
        factory = templateNode.getProject().getNodeFactory();
        factory.importFile(templateNode.getProject(),
                           templateNode.getOutputFile());
    }

    /**
     * Method declaration
     *
     *
     * @param confNode
     *
     * @see
     */
    private void processNode(OtterTemplateNode templateNode) {
        TemplateFile tf = new TemplateFile(templateNode);

        try {
            tf.backupOutFile();
            tf.deleteOutFile();
            tf.copyInFileToOutFile();
            tf.processOutFile();
        } catch (java.io.FileNotFoundException e) {
            templateNode.setException(e);
        } catch (java.io.IOException e) {
            templateNode.setException(e);
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision$
 */
class TemplateFile {
    private File       inFile = null;
    private File       outFile = null;
    private String[][] confTable = new String[0][0];

    /**
     * Constructor declaration
     *
     *
     * @param confNode
     *
     * @see
     */
    protected TemplateFile(OtterTemplateNode templateNode) {
        OtterProject project = templateNode.getProject();
        String[][]   rawTable = project.getConfTable();

        inFile = new File(templateNode.getFilePath());
        outFile = templateNode.getOutputFile();
        confTable = new String[rawTable.length][2];
        for (int i = 0; i < rawTable.length; i++) {
            confTable[i][0] = rawTable[i][0];
            confTable[i][1] = PropUtil.expandPathRelativeToProject(project,
                    rawTable[i][1]);
        }

        // make any needed directories.
        outFile.getParentFile().mkdirs();
    }

    /**
     * Method declaration
     *
     *
     * @exception java.io.FileNotFoundException
     * @exception java.io.IOException
     *
     * @see
     */
    protected void backupOutFile()
            throws java.io.FileNotFoundException, java.io.IOException {
        if (outFile.exists()) {
            FileUtil.copyFile(outFile,
                              new File(outFile.getAbsolutePath() + ".bak"));
        }
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    protected void deleteOutFile() {
        if (outFile.exists()) {
            outFile.delete();
        }
    }

    /**
     * Method declaration
     *
     *
     * @exception java.io.FileNotFoundException
     * @exception java.io.IOException
     *
     * @see
     */
    protected void copyInFileToOutFile()
            throws java.io.FileNotFoundException, java.io.IOException {
        FileUtil.copyFile(inFile, outFile);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    protected void processOutFile() {
        String[] find = new String[confTable.length];
        String[] replace = new String[confTable.length];
        boolean[] longFile = new boolean[confTable.length];

        for (int i = 0; i < confTable.length; i++) {
            find[i] = confTable[i][0];
            replace[i] = confTable[i][1];
            longFile[i] = false;
            // if it has embedded spaces check if it may be a file.
            if (replace[i].trim().indexOf(" ") > 0) {
              File file = new File(replace[i]);
              if (file.getParentFile() != null) {
                 if (file.getParentFile().getParentFile() != null) {
                    if (file.getParentFile().getParentFile().exists()) {
                      replace[i] = "\"" + replace[i] + "\"";
                      replace[i] = replace[i].replace('\\','/');
                      longFile[i] = true;
                    }
                 }
              }
            }
        }
        searchAndReplace(outFile.getAbsolutePath(), find, replace, longFile);
    }

    /**
     * Method declaration
     *
     *
     * @param filename
     * @param find
     * @param replace
     *
     * @see
     */
    private void searchAndReplace(String filename, String[] find,
                                  String[] replace,
                                  boolean[] longFile) {
        BufferedReader in = null;
        PrintWriter    out = null;
        File           file = new File(filename);
        File           temp = null;

        try {
            temp = File.createTempFile("kelp", ".tmp",
                                       new File(file.getParent()));
            String tempName = temp.getAbsolutePath();

            temp.delete();
            temp = null;
            file.renameTo(new File(tempName));
            file = null;
            temp = new File(tempName);
            in = new BufferedReader(new FileReader(temp));
            out = new PrintWriter(new FileWriter(new File(filename)));
            String line = in.readLine();

            while (line != null) {
                line = lineSearchAndReplace(line,
                                            createReplacementSet(find,
                                            replace, longFile));
                out.println(line);
                line = in.readLine();
            }
            in.close();
            out.close();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
        temp.delete();
        in = null;
        out = null;
        file = null;
        temp = null;
    }

    /**
     * Method declaration
     *
     *
     * @param line
     *
     * @return
     *
     * @see
     */

    // See org.enhydra.codegen.TemplateDrivenGenerator::lineSearchAndReplace()
    private String lineSearchAndReplace(String in,
                                        LocalReplacement[] replacements) {
        String text = in;
        String prefix = new String();
        String suffix = new String();

        for (int i = 0; i < replacements.length; i++) {
            int indexOf = text.indexOf(replacements[i].getFind());
            int lastIndex = -1;
            int count = 0;

            while (indexOf != -1 && indexOf > lastIndex && count < 1000) {

                // Catch recursive replacements by maxing out at
                // after the 1000 replacement
                count++;
                prefix = text.substring(0, indexOf);
                suffix = text.substring(indexOf
                                        + replacements[i].getFind().length());
                StringBuffer buf = new StringBuffer();

                buf.append(prefix);
                for (int j = 0; j < replacements[i].getReplaceWith().length;
                        j++) {
                    String rep = replacements[i].getReplaceWith()[j];
                    if (replacements[i].isLongFileName()) {
                      if (suffix.startsWith("/")
                        && rep.endsWith("\"")) {
                        suffix = suffix + "\"";
                        rep = rep.substring(0, (rep.length()-1) );
                      }
                    }
                    buf.append(rep);
                }
                buf.append(suffix);
                text = buf.toString();
                indexOf = text.indexOf(replacements[i].getFind());
            }
        }
        return text;
    }

    /**
     * Method declaration
     *
     *
     * @param prefix
     * @param replacement
     * @param suffix
     *
     * @return
     *
     * @see
     */
    private String transformFileValues(String prefix, String replacement,
                                       String suffix) {

        // Quote and use unix style separators
        // to support long file names on Windows.
        String line = prefix + replacement + suffix;
        String value = replacement + suffix;
        File   file;

        if (prefix.trim().endsWith("=")) {
            file = new File(replacement);
            file = new File(file.getParent());
            file = new File(file.getParent());
            if (file.exists() && file.isDirectory()) {
                value = value.replace('\\', '/');
                value = "\"" + value + "\"";
            }
            line = prefix + value;
        }
        return line;
    }

    /**
     * Method declaration
     *
     *
     * @param find
     * @param replaceWith
     *
     * @return
     *
     * @see
     */
    private LocalReplacement[] createReplacementSet(String[] find,
            String[] replaceWith, boolean[] longFile) {
        LocalReplacement[] replacements = new LocalReplacement[find.length];

        for (int i = 0; i < find.length; i++) {
            replacements[i] = new LocalReplacement(find[i], replaceWith[i], longFile[i]);
        }
        return replacements;
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision$
     */
    class LocalReplacement {
        private String   find = new String();
        private String[] replaceWith = new String[1];
        private boolean longFile = false;

        /**
         * Constructor declaration
         *
         *
         * @param find
         * @param replaceWith
         *
         * @see
         */
        private LocalReplacement(String f, String r, boolean b) {
            find = f;
            replaceWith[0] = r;
            longFile = b;
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        private String getFind() {
            return find;
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        private String[] getReplaceWith() {
            return replaceWith;
        }

        private boolean isLongFileName() {
          return longFile;
        }

    }
}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision$
 */
class StartMultiServer {
    private OtterProject project = null;
    private File         directory;
    private File         file = null;

    /**
     * Constructor declaration
     *
     *
     * @param p
     *
     * @see
     */
    protected StartMultiServer(OtterProject p) {
        project = p;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected File getDirectory() {
        if (directory == null) {
            String[] paths = project.getSourcePathArray();

            directory = new File(paths[0]);
        }
        return directory;
    }

    /**
     * Method declaration
     *
     *
     * @param d
     *
     * @see
     */
    protected void setDirectory(File d) {
        if (d != null) {
            directory = d;
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected File getFile() {
        String filename = "StartMultiServer.java";

        file = new File(getDirectory(), filename);
        return file;
    }

    /**
     * Write a StartMultisServer.java file for launching and stopping MultiServer
     * with a simple graphical interface. This is useful in environments such as
     * the Win32 version of JBuilder 3 where you cannot run an arbitrary class
     * from a project.
     *
     * @throws IOException
     */
    public void create() throws java.io.IOException {
        if ((!getFile().exists()) || project.isConfOverwriteSelected()) {
            PrintWriter out = new PrintWriter(new FileWriter(getFile()));

            StringBuffer startBuf = new StringBuffer();
            startBuf.append(project.getConfStartupConf());
            if (! KelpVersion.getEnhydraVersion().startsWith("3")) {
               startBuf.append(File.separator);
               startBuf.append("servlet");
               startBuf.append(File.separator);
               startBuf.append("kernel.conf");
            }

            out.println("");
            out.println("import java.awt.*;");
            out.println("import java.awt.event.*;");
            out.println("");
            out.println("public class StartMultiServer extends Frame {");
            out.println("  Button button = new Button();");
            out.println("");
            out.println("  public static void main(String[] args) {");
            out.println("    if (args == null || args.length < 1) {");
            out.println("      args = new String[1];");
            out.println("      args[0] = \""
                        + doubleEscape(startBuf.toString()) + "\";");
            out.println("     }");
            out.println("    " + KelpVersion.getMultiServerRunClass()
                        + ".main(args);");
            out.println("    System.setSecurityManager(null);");
            out.println("    StartMultiServer frame = new StartMultiServer();");
            out.println("    frame.resize(200,100);");
            out.println("    frame.setLocation(new Point(200, 200));");
            out.println("    frame.show();");
            out.println("    frame.setLocation(new Point(200, 200)); // reset for Linux AWT");
            out.println("  }");
            out.println("");
            out.println("    public StartMultiServer() {");
            out.println("      button.setLabel(\"Shutdown MultiServer\");");
            out.println("      button.addActionListener(new ActionListener() {");
            out.println("        public void actionPerformed(ActionEvent e) {");
            out.println("         System.exit(0);");
            out.println("        }");
            out.println("      });");
            out.println("      add(button);");
            out.println("      enableEvents(java.awt.AWTEvent.WINDOW_EVENT_MASK);");
            out.println("      setTitle(\"MultiServer\");");
            out.println("    }");
            out.println("");
            out.println("  protected void processWindowEvent(java.awt.event.WindowEvent e) {");
            out.println("    super.processWindowEvent(e);");
            out.println("    if (e.getID() == java.awt.event.WindowEvent.WINDOW_CLOSING) {");
            out.println("      System.exit(0);");
            out.println("    }");
            out.println("  }");
            out.println("");
            out.println("}");
            out.println("");
            out.flush();
            out.close();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param raw
     *
     * @return
     *
     * @see
     */
    private String doubleEscape(String raw) {
        StringBuffer doubled = new StringBuffer();
        String       in = new String(raw);
        int          index = in.indexOf("\\");

        while (index > -1) {
            doubled.append(in.substring(0, index));
            doubled.append("\\\\");
            if (index == in.lastIndexOf("\\")) {
                if (!in.endsWith("\\")) {
                    doubled.append(in.substring(index + 1));
                }
                in = "";
            } else {
                in = in.substring(index + 1);
            }
            index = in.indexOf("\\");
        }
        if (doubled.toString().trim().length() == 0) {

            // Should not be any "\" on Linux so
            // no need to double escape.
            doubled.append(raw);
        }
        return doubled.toString();
    }

}
