/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.common.wizard.xmlc;
// Kelp imports
import org.enhydra.kelp.common.wizard.AbstractPropertyPanel;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.SwingUtil;
// Standard imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;


/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class XMLCPropertyPanel extends AbstractPropertyPanel {
    private final String        mapAll = "All";
    private final String        mapSelected = "Selected";
    private final String        mapNone = "None";

    private LocalButtonListener buttonListener = null;
    private LocalRadioListener radioListener = null;

    private OtterProject        project = null;
    private boolean             active = false;
    private GridBagLayout       layoutCompile;
    private JPanel              panelJavaSource;
    private GridBagLayout       layoutJavaSource;
    private JCheckBox           checkKeep;
    private JCheckBox           checkCompile;
    private TitledBorder        borderJavaSource;
    private TitledBorder        borderMappings;
    private TitledBorder        borderResources;
    private JPanel              panelMappings;
    private GridBagLayout       layoutMap;
    private JButton             buttonMap;
    private JPanel              panelTrace;
    private JPanel              panelResources;
    private GridBagLayout       layoutTrace;
    private JCheckBox           checkParseInfo;
    private JCheckBox           checkPrintAccessMethods;
    private JCheckBox           checkPrintInfo;
    private JCheckBox           checkDump;
    private JCheckBox           checkVerbose;
    private JCheckBox           checkAddToProject;
    private JComboBox           comboMap;
    private JLabel              labelMap;
    private XMLCOptionPanel     optionPanel;
    private JPanel              panelMake;
    private JCheckBox           checkMake;
    private GridBagLayout       layoutMake;
    private JTabbedPane         tabPane;
    private JPanel              panelCompile;
    private BorderLayout        layoutMain;
    private JPanel              panelBorderResources;
    private GridBagLayout       layoutResourcesMain;
    private GridBagLayout       layoutResources;
    private JRadioButton        radioResourceMap;
    private JRadioButton        radioResourcePath;
    private JLabel              labelResourcePath;
    private JTextField          textResourcePath;
    private JButton             buttonSetResourcePath;
    private ButtonGroup         radioGroup;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCPropertyPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setProject(OtterProject p) {
        project = p;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void readProperties() {
        optionPanel.setNode(project);
        optionPanel.readProperties();
        if (comboMap.getItemCount() > 0) {
            comboMap.removeAllItems();
        }
        checkCompile.setEnabled(false);
        checkCompile.setSelected(false);
        if (project.isOpenBuild()) {

            // Primetime
            checkMake.setEnabled(true);
            checkMake.setSelected(project.isXMLCMake());
            comboMap.addItem(mapAll);
            comboMap.addItem(mapSelected);
            comboMap.addItem(mapNone);
            switch (project.getMapScope()) {
            case OtterProject.MAP_SCOPE_NONE:
                comboMap.setSelectedItem(mapNone);
                break;
            case OtterProject.MAP_SCOPE_SELECTED:
                comboMap.setSelectedItem(mapSelected);
                break;
            default:
                project.setMapScope(OtterProject.MAP_SCOPE_ALL);
                comboMap.setSelectedItem(mapAll);
                break;
            }
            checkAddToProject.setEnabled(false);
            checkAddToProject.setSelected(true);
        } else {

            // Win32 and JDeveloper
            checkMake.setEnabled(false);
            checkMake.setSelected(false);
            comboMap.addItem(mapAll);
            comboMap.addItem(mapNone);
            if (project.getMapScope() == OtterProject.MAP_SCOPE_NONE) {
                comboMap.setSelectedItem(mapNone);
            } else {
                comboMap.setSelectedItem(mapAll);
                project.setMapScope(OtterProject.MAP_SCOPE_ALL);
            }
            checkAddToProject.setEnabled(false);
            checkAddToProject.setSelected(false);
        }
        if (project.isResourceMap()) {
            radioResourceMap.setSelected(true);
            radioResourcePath.setSelected(false);
            textResourcePath.setText("");
            textResourcePath.setEnabled(false);
            buttonSetResourcePath.setEnabled(false);
        } else {
            radioResourceMap.setSelected(false);
            radioResourcePath.setSelected(true);
            textResourcePath.setText(project.getResourcePath());
            textResourcePath.setEnabled(true);
            buttonSetResourcePath.setEnabled(true);
        }
        checkDump.setSelected(project.isDumpSelected());
        checkParseInfo.setSelected(project.isParseInfoSelected());
        checkPrintAccessMethods.setSelected(project.isPrintAccessMethodsSelected());
        checkPrintInfo.setSelected(project.isPrintInfoSelected());
        checkVerbose.setSelected(project.isVerboseSelected());
        active = true;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void writeProperties() {
        String item = (String) comboMap.getSelectedItem();

        item = item.trim();
        optionPanel.writeProperties();
        if (project != null && active) {
            if (item.equalsIgnoreCase(mapAll)) {
                project.setMapScope(OtterProject.MAP_SCOPE_ALL);
            } else if (item.equalsIgnoreCase(mapSelected)) {
                project.setMapScope(OtterProject.MAP_SCOPE_SELECTED);
            } else if (item.equalsIgnoreCase(mapNone)) {
                project.setMapScope(OtterProject.MAP_SCOPE_NONE);
            }
        }
        if (project.isOpenBuild()) {

            // Primetime
            project.setXMLCMake(checkMake.isSelected());
        } else {

            // Win32 and JDeveloper
            project.setXMLCMake(false);
        }
        if (radioResourceMap.isSelected()) {
            project.setResourceMap(true);
            project.setResourcePath("");
        } else {
            project.setResourceMap(false);
            project.setResourcePath(textResourcePath.getText());
        }
        project.setDumpSelected(checkDump.isSelected());
        project.setParseInfoSelected(checkParseInfo.isSelected());
        project.setPrintAccessMethodsSelected(checkPrintAccessMethods.isSelected());
        project.setPrintInfoSelected(checkPrintInfo.isSelected());
        project.setVerboseSelected(checkVerbose.isSelected());
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
        checkKeep.setSelected(true);
        checkKeep.setEnabled(false);
        buttonListener = new LocalButtonListener();
        buttonMap.addActionListener(buttonListener);
        buttonSetResourcePath.addActionListener(buttonListener);

        radioGroup = new ButtonGroup();
        radioGroup.add(radioResourceMap);
        radioGroup.add(radioResourcePath);

        radioListener = new LocalRadioListener();
        radioResourceMap.addActionListener(radioListener);
        radioResourcePath.addActionListener(radioListener);
    }

    private void browseForResourcePath() {
      File choice = null;
      choice = SwingUtil.getDirectoryChoice(this, textResourcePath.getText(),
	  "Select the resource root document path");
      if (choice != null && choice.isDirectory()) {
         textResourcePath.setText(choice.getAbsolutePath());
      }

    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
        layoutCompile =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        optionPanel =
            (XMLCOptionPanel) Beans.instantiate(getClass().getClassLoader(),
                                                XMLCOptionPanel.class.getName());
        panelJavaSource =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutJavaSource =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkKeep = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        checkCompile =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        borderJavaSource = new TitledBorder("");
        borderMappings = new TitledBorder("");
        borderResources = new TitledBorder("Static resource path");
        panelMappings =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutMap =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonMap = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        panelTrace = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());
        panelResources =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutTrace =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkParseInfo =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkPrintAccessMethods =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkPrintInfo =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkDump = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        checkVerbose =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkAddToProject =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        comboMap = (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                                 JComboBox.class.getName());
        labelMap = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                              JLabel.class.getName());
        panelMake = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        checkMake = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        layoutMake =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        tabPane =
            (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                            JTabbedPane.class.getName());
        panelCompile = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        panelBorderResources =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutResourcesMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutResources =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        radioResourceMap =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioResourcePath =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        labelResourcePath =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        textResourcePath =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonSetResourcePath =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        borderJavaSource.setTitle("Generated Java Source");
        borderMappings.setTitle("Mappings");
        panelJavaSource.setLayout(layoutJavaSource);
        panelJavaSource.setBorder(borderJavaSource);
        panelMappings.setBorder(borderMappings);
        panelTrace.setLayout(layoutTrace);
        checkKeep.setSelected(true);
        checkKeep.setText("Keep");
        checkCompile.setSelected(true);
        checkCompile.setText("Compile");
        panelMappings.setLayout(layoutMap);
        buttonMap.setText("Edit...");
        checkParseInfo.setText("Parsing details");
        checkParseInfo.setActionCommand("");
        checkPrintInfo.setText("IDs and URLs");
        checkPrintInfo.setActionCommand("Show information about the page object, such as ids and URLs");
        checkDump.setText("DOM tree");
        checkDump.setActionCommand("");
        checkVerbose.setActionCommand("");
        checkVerbose.setText("Compile process");
        checkPrintAccessMethods.setText("Access methods");
        checkAddToProject.setText("Add to Project");
        checkAddToProject.setActionCommand("");
        labelMap.setHorizontalAlignment(SwingConstants.RIGHT);
        labelMap.setText("Map Nodes:");
        checkMake.setEnabled(false);
        checkMake.setText("Invoke XMLC during Project Make/Rebuild");
        panelMake.setLayout(layoutMake);
        this.setLayout(layoutMain);
        panelResources.setLayout(layoutResourcesMain);
        panelBorderResources.setLayout(layoutResources);
        panelBorderResources.setBorder(borderResources);
    panelBorderResources.setMaximumSize(new Dimension(200, 130));
    panelBorderResources.setMinimumSize(new Dimension(200, 130));
    panelBorderResources.setPreferredSize(new Dimension(200, 130));
        radioResourceMap.setText("Use package map with source path");
        radioResourcePath.setText("Set resource document root path");
        labelResourcePath.setText("Path:");
        textResourcePath.setMaximumSize(new Dimension(110, 21));
    textResourcePath.setMinimumSize(new Dimension(110, 21));
    textResourcePath.setPreferredSize(new Dimension(110, 21));
    textResourcePath.setText("jTextField1");
        buttonSetResourcePath.setText("...");
        panelMake.add(checkMake,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 2, 2, 2), 0, 0));
        panelJavaSource.add(checkKeep,
                            new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.HORIZONTAL,
                                                   new Insets(2, 2, 2, 2), 5,
                                                   0));
        panelJavaSource.add(checkAddToProject,
                            new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.HORIZONTAL,
                                                   new Insets(2, 2, 2, 2), 5,
                                                   0));
        panelJavaSource.add(checkCompile,
                            new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.HORIZONTAL,
                                                   new Insets(2, 2, 2, 2), 5,
                                                   0));
        panelMappings.add(buttonMap,
                          new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.NONE,
                                                 new Insets(2, 2, 1, 2), 0,
                                                 0));
        panelMappings.add(comboMap,
                          new GridBagConstraints(1, 0, 1, 1, 0.2, 0.2,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.HORIZONTAL,
                                                 new Insets(2, 2, 1, 2), 0,
                                                 0));
        panelMappings.add(labelMap,
                          new GridBagConstraints(0, 0, 1, 1, 0.2, 0.2,
                                                 GridBagConstraints.EAST,
                                                 GridBagConstraints.HORIZONTAL,
                                                 new Insets(2, 2, 1, 2), 0,
                                                 0));
        panelCompile.setLayout(layoutCompile);
        panelCompile.add(panelJavaSource,
                         new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(2, 2, 2, 2), 5,
                                                5));
        panelCompile.add(panelMappings,
                         new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(2, 2, 2, 0), 5,
                                                5));
        panelCompile.add(optionPanel,
                         new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(1, 2, 2, 2), 0,
                                                0));
        panelTrace.add(checkPrintInfo,
                       new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(0, 0, 0, 0), 0, 0));
        panelTrace.add(checkParseInfo,
                       new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(2, 2, 2, 2), 5, 0));
        panelTrace.add(checkDump,
                       new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(2, 2, 2, 2), 5, 0));
        panelTrace.add(checkVerbose,
                       new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.CENTER,
                                              GridBagConstraints.NONE,
                                              new Insets(2, 2, 2, 2), 5, 0));
        panelTrace.add(checkPrintAccessMethods,
                       new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(2, 2, 2, 2), 5, 0));
        tabPane.add(panelCompile, "Compile");
        tabPane.add(panelTrace, "Trace");
        tabPane.add(panelResources, "Resources");
        panelResources.add(panelBorderResources, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(20, 20, 20, 20), 220, 100));
        panelBorderResources.add(radioResourceMap, new GridBagConstraints(0, 0, 3, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 10, 5, 10), 0, 0));
        panelBorderResources.add(radioResourcePath, new GridBagConstraints(0, 1, 3, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 10, 5, 10), 0, 0));
        panelBorderResources.add(labelResourcePath, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2, 25, 5, 2), 0, 0));
        panelBorderResources.add(textResourcePath, new GridBagConstraints(1, 2, 1, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 2, 5, 2), 110, 0));
        panelBorderResources.add(buttonSetResourcePath, new GridBagConstraints(2, 2, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 5, 2), 0, 0));
        this.add(tabPane, BorderLayout.CENTER);
        this.add(panelMake, BorderLayout.NORTH);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void editProjectMap() {
        MapDialog dialog =
            new MapDialog(((JDialog) this.getTopLevelAncestor()),
                          "Edit Project Map", true);

        dialog.setProject(project);
        dialog.show();
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

        /**
         * Method declaration
         *
         *
         * @param event
         *
         * @see
         */
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonMap) {
                editProjectMap();
            } else if (source == buttonSetResourcePath) {
               browseForResourcePath();
            }
        }

    }


    class LocalRadioListener implements ActionListener {

        /**
         * Method declaration
         *
         *
         * @param event
         *
         * @see
         */
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (radioResourceMap.isSelected()) {
              textResourcePath.setText("");
              textResourcePath.setEnabled(false);
              buttonSetResourcePath.setEnabled(false);
            } else {
              textResourcePath.setText(project.getSourcePathArray()[0]);
              textResourcePath.setEnabled(true);
              buttonSetResourcePath.setEnabled(true);
            }
        }

    }

}
