/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.jbuilder.wizard;

import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeHTMLFileNode;
import org.enhydra.kelp.common.wizard.xmlc.XMLCBuildDialog;
import org.enhydra.kelp.common.node.OtterXMLCNode;

import com.borland.jbuilder.node.JBProject;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserListener;
import com.borland.primetime.ide.NodeViewer;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.wizard.BasicWizard;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.vfs.InvalidUrlException;
import javax.swing.JOptionPane;
import java.io.File;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
abstract public class AbstractBasicWizard extends BasicWizard {
    private Vector	       sourceVector = new Vector();
    private JBProject	       nativeProject = null;
    public static boolean enableMenu = true;

    public static BrowserListener browserListener =
        new AbstractBasicWizardBrowserListener();

    public JBProject getNativeProject() {
      return nativeProject;
    }

    public void setNativeProject(JBProject p) {
      nativeProject = p;
    }

    public Vector getSourceVector() {
      return sourceVector;
    }
    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @return
     *
     * @see
     */
    public WizardPage invokeWizard(WizardHost host) {
	setWizardTitle(getName());
        InvokerThread invoker = new InvokerThread(host);
        invoker.run();
	return super.invokeWizard(host);
    }

    /**
     * Method declaration
     *
     *
     * @throws VetoException
     *
     * @see
     */
    protected void finish() throws VetoException {}

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public static String getName() {
      return "Enhydra Wizard";
    }

    public static class AbstractBasicWizardBrowserListener implements BrowserListener {
      public void browserActivated(Browser browser) {
        refreshMenu(browser);
      }
      public void browserClosed(Browser browser) {
        refreshMenu(browser);
      }
      public void browserClosing(Browser browser) throws VetoException {
        refreshMenu(browser);
      }
      public void browserDeactivated(Browser browser) {
        refreshMenu(browser);
      }
      public void browserOpened(Browser browser) {
        refreshMenu(browser);
      }
      public void browserProjectActivated(Browser browser, Project project) {
        refreshMenu(browser);
      }
      public void browserProjectClosed(Browser browser, Project project) {
        refreshMenu(browser);
      }
      public void browserNodeActivated(Browser browser, Node node) {
        refreshMenu(browser);
      }
      public void browserNodeClosed(Browser browser, Node node) {
        refreshMenu(browser);
      }
      public void browserViewerActivated(Browser b, Node n, NodeViewer v) {
        refreshMenu(b);
      }
      public void browserViewerDeactivating(Browser b, Node n, NodeViewer v)
           throws VetoException {
        refreshMenu(b);
      }

      private void refreshMenu(Browser b) {
        Project ap = b.getActiveProject();
        AbstractBasicWizard.enableMenu = (ap != null);
      }

    }


    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @see
     */
    abstract public void invoke(WizardHost host);

    /**
     * Method declaration
     *
     *
     * @param projectClassPath
     *
     * @return
     *
     * @see
     */
    public boolean isEnhydraAvailable(String projectClassPath) {
	String message = null;

	if (! KelpVersion.isEnhydraInClassPath()) {
	    message = "Enhydra not found in IDE class path. \n"
		      + "Please verify that the enhydra.jar file "
                      + "is included your IDE class path.";
	}

	if (message == null) {
    	   if (!nativeProject.getPaths().putClassOnFullPath(
                     KelpVersion.XMLC_RUN_CLASS)) {
		message =
		    "The enhydra.jar file is not included in the "
		    + "current project library set.\n"
		    + "Select the Project | Properties menu "
		    + "to add it before using the Kelp wizards.\n\n";
	    }
	}

	if (message != null) {
	    JOptionPane.showMessageDialog(null, message);
	}

	return (message == null);
    }


    class InvokerThread extends Thread {
        private WizardHost host;

        public InvokerThread(WizardHost h) {
          host = h;
        }

        public void run() {
      	   invoke(host);
        }
    }

}
