/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.jbuilder.wizard.conf;

import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.jbuilder.wizard.AbstractBasicWizard;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeTemplateNode;
import org.enhydra.kelp.common.wizard.conf.ConfBuildDialog;
import org.enhydra.kelp.common.node.OtterTemplateNode;

import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserListener;
import com.borland.primetime.ide.NodeViewer;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.vfs.InvalidUrlException;
import javax.swing.JOptionPane;
import java.io.File;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class ConfWizard extends AbstractBasicWizard {

    public static WizardAction action =  new ConfWizardAction();
    /**
     * Method declaration
     *
     *
     * @param majorVersion
     * @param minorVersion
     *
     * @see
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
	WizardManager.registerWizardAction(action);
        Browser.addStaticBrowserListener(browserListener);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public static String getName() {
	return "Local Deployment";
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    static class ConfWizardAction extends WizardAction {

	/**
	 * Constructor declaration
	 *
	 *
	 * @see
	 */
	public ConfWizardAction() {
	    super(getName() + "...", 'X',
		  "Process template files (*.xml.in, *.conf.in)",
                  IconSet.getSmallIcon(),
                  IconSet.getLargeIcon(),
		  false);
        }

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	protected Wizard createWizard() {
	    return new ConfWizard();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param source
	 *
	 * @see
	 */
	public void update(Object source) {
	    setEnabled(AbstractBasicWizard.enableMenu);
	}

    }

    ;

    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @see
     */
    public synchronized void invoke(WizardHost host) {
	PrimeProject         primeProject = null;
	OtterTemplateNode[]  templateNodes    = null;
	ConfBuildDialog dlg = null;
	setNativeProject(
          (JBProject) host.getBrowser().getProjectView().getActiveProject());

	if (getNativeProject() != null) {
	    try {
		getNativeProject().save();
	    } catch (InvalidUrlException e) {
		e.printStackTrace();
	    } catch (java.io.IOException e) {
		e.printStackTrace();
	    }

	    primeProject = new PrimeProject(getNativeProject());
	}
	if (isEnhydraAvailable(primeProject.getClassPath())) {
	    dlg = new ConfBuildDialog(host.getDialogParent(), getName(), true);
	    readNodes(getNativeProject());
	    templateNodes = new OtterTemplateNode[getSourceVector().size()];

	    for (int i = 0; i < templateNodes.length; i++) {
		templateNodes[i] = (OtterTemplateNode) getSourceVector().elementAt(i);
	    }

	    dlg.setProject(primeProject);
	    dlg.setNodes(templateNodes);
	    dlg.show();
	}
    }

    // recursively read through nodes.

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void readNodes(Node container) {
	String        filename = null;
	OtterTemplateNode templateNode = null;
	Node[]        nativeNodes = container.getChildren();
	FileNode      fileNode = null;
	File	      current = null;
	String        projectPage = getNativeProject().getUrl().getFile();

	for (int i = 0; i < nativeNodes.length; i++) {
	    if (nativeNodes[i] instanceof FileNode) {
		fileNode = (FileNode) nativeNodes[i];
		filename = fileNode.getUrl().getFile();
		current = new File(filename);
                filename = current.getAbsolutePath();
		if (current.exists() &&
		   (filename.toLowerCase().endsWith(".conf.in") ||
                    filename.toLowerCase().endsWith(".xml.in"))) {
          	    PrimeTemplateNode primeNode = null;
                    templateNode = new PrimeTemplateNode((TextFileNode)fileNode);
  	            getSourceVector().addElement(templateNode);
		}
	    }

	    readNodes(nativeNodes[i]);    // recrusive folder read.
	}
    }

}
