/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */


package org.enhydra.kelp.jdeveloper.node;

// Kelp imports
import org.enhydra.kelp.common.node.OtterHTMLFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterResourceNode;
import org.enhydra.kelp.common.node.OtterSelectionNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.PropUtil;
// JDeveloper imports
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.HtmlNode;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.Url;
// Standard imports
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class OracleHTMLFileNode extends OracleNode implements OtterHTMLFileNode {
    private Exception exception = null;

    public OracleHTMLFileNode(HtmlNode n) {
        super((Node)n);
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param file
     *
     * @see
     */
    public OracleHTMLFileNode(OtterProject otterProject, OtterNode otterNode,
			   String file) {
	Node    nativeNode = null;
	Project nativeProject = null;
	Url     url = null;

	if (otterProject.getNativeProject() instanceof Project) {
	    nativeProject = (Project) otterProject.getNativeProject();
	}

	url = new Url(file);
	nativeNode = nativeProject.createNode(url, true);

	nativeProject.addNode(nativeNode);
	setNativeNode(nativeNode);
    }

  public boolean isSelected() {
   String  in = getProperty(OtterSelectionNode.SELECTED);
   return PropUtil.stringToBoolean(in, false);
  }

  public void setSelected(boolean b) {
	setProperty(OtterSelectionNode.SELECTED,
	   PropUtil.booleanToString(b));
        if (b) {
          setStatic(false);
          // both may be false, but only can one true
        }
  }

  public boolean isStatic() {
   String  in = getProperty(OtterResourceNode.RESOURCE_STATIC);
   return PropUtil.stringToBoolean(in, false);
  }

  public void setStatic(boolean b) {
	setProperty(OtterResourceNode.RESOURCE_STATIC,
	   PropUtil.booleanToString(b));
        if (b) {
          setSelected(false);
        }
  }


    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
	exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
	return exception;
    }



  public void preCompile() {
    // nothing to do
  }

}
